/*
 *-----------------------------------------------------------------------------
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from Arm Limited or its affiliates.
 *
 *            (C) COPYRIGHT 2010-2013 Arm Limited or its affiliates.
 *                ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from Arm Limited or its affiliates.
 *
 *      SVN Information
 *
 *      Checked In          : $Date: 2017-10-10 15:55:38 +0100 (Tue, 10 Oct 2017) $
 *
 *      Revision            : $Revision: 371321 $
 *
 *      Release Information : Cortex-M System Design Kit-r1p1-00rel0
 *-----------------------------------------------------------------------------
 */

//==============================================================================
//
// Cortex-M0+ System Design Kit USER Test
//
//==============================================================================
//
// This test checks:
//
// USER
//
//   -  The presence of USER mode support is checked by writing and reading
//      the nPRIV bit of special CONTROL register.
//   -  If USER mode support is detected, try to trigger HardFault by
//      accessing NVIC in unprivileged state.
//   -  Check the configurations in CMSIS header and config_id.h
//      to make sure settings are consistence with RTL

#ifdef CORTEX_M0
#include "CMSDK_CM0.h"
#include "core_cm0.h"
#endif

#ifdef CORTEX_M0PLUS
#include "CMSDK_CM0plus.h"
#include "core_cm0plus.h"
#endif

#include <stdio.h>
#include "uart_stdout.h"
#include "CMSDK_driver.h"

// This program checks the EXPECTED_MPU and EXPECTED_USER parameters in
// "config_id.h" to make sure that it matches the hardware/RTL configuration.
#include "config_id.h"

//=====================================================================
//Function prototype
//=====================================================================
void Check_USER(void);

//=====================================================================
// Global Variables to track test progress
//=====================================================================
         uint32_t Errors = 0;
volatile uint32_t user_support;           // hold result of configuration detection
volatile uint32_t HardFault_expected = 0; // Indicate to HardFault handler is the
                                          // fault is expected (part of the test)
volatile uint32_t HardFault_occurred = 0; // Variable to indicate HardFault has
                                          // been triggered

int main (void)
{
  // UART init
  UartStdOutInit();

#ifdef CORTEX_M0PLUS
  // Banner
  puts("Cortex-M0+ - USER testing\n");
  Check_USER();

  //Print final test result
  if (Errors == 0) {
     puts("** TEST PASSED ** \n");
  } else {
     printf("** TEST FAILED ** with Errors code:%d\n", Errors);
  }
#else
  puts( "** TEST SKIPPED ** Core does not support USER, skipping test\n");
#endif

  // End simulation
  UartEndSimulation();

  return Errors;
}

#ifdef CORTEX_M0PLUS
//
// Checking Routines
//

void Check_USER()
{
  //
  // USER
  //
  // Determine whether USER mode is present by writing and reading CONTROL.nPRIV
  // If write is successful, USER mode is present and perform NVIC access to
  // show code running in user state cannot access to certain resources such as NVIC
  //


  // Read CONTROL nPRIV and check
  puts   ("- Set CONTROL[0] to 1");

  // Try to set USER mode
  __set_CONTROL(1);
  __ISB();

  user_support = __get_CONTROL();
  printf ("- CONTROL = %d\n", user_support);

  if (user_support == 1) {
    puts ("USER support detected\n"); // Note: CONTROL[0] is now 1, (unprivileged)

    puts ("Test NVIC access in unprivileged state, HardFault expected...");

    // Set variable flag to indicate HardFault is expected
    HardFault_expected = 1;

    NVIC->ICER[0] = 0; // Test access to NVIC, should not be allowed
    __DSB();

    if (HardFault_occurred==0) {
      puts ("ERROR: HardFault didn't take place.\nTEST FAILED\n");
      Errors |= 1;
      }
    else {
      puts ("HardFault taken place successfully");
      }
    HardFault_expected = 0;
    }
  else {
    puts ("USER support NOT detected");
    // NVIC access test not needed
    }

  // Configuration checks
  puts ("\n\nConfiguration checks:");

  // Check settings in ../generic/config_id.h

  if ((user_support == 1) && (EXPECTED_USER==0)) {
    puts ("ERROR: EXPECTED_USER settings in ../generic/config_id.h does not\n");
    puts ("       match detected configuration.\n");
    Errors |= 1<<1;
    }
  if ((user_support == 0) && (EXPECTED_MPU==8)) {
    puts ("ERROR: EXPECTED_MPU configuration should not be 8 when detected configuration");
    puts ("       showing USER option not set.\n");
    Errors |= 1<<2;
    }

  if ((EXPECTED_USER == 0) && (EXPECTED_MPU==8)) {
    puts ("ERROR: EXPECTED_MPU configuration should not be 8 when EXPECTED_USER option not set.\n");
    Errors |= 1<<3;
    }

  // Check __MPU_PRESENT setting in CMSIS header file

  if ( ((__MPU_PRESENT!=0) && (EXPECTED_MPU==0)) || ((__MPU_PRESENT==0) && (EXPECTED_MPU==8)) ) {
    printf ("ERROR: EXPECTED_MPU (%d) configuration does not match __MPU_PRESENT (%d)\n", EXPECTED_MPU, __MPU_PRESENT);
    Errors |= 1<<4;
    }

  return;
}


// HardFault handler - NVIC access in unprivileged execution state results in HardFault
void HardFault_Handler(void)
{
  puts ("[HardFault]");
  if (HardFault_expected==1) {
    // HardFault is expected
    __set_CONTROL(0);       // Switch back to Privileged Thread
    HardFault_occurred = 1; // Update software flag
    }
  else {
    puts ("ERROR: HardFault is unexpected.\nTEST FAILED\n");
    // End simulation
    UartEndSimulation();
    }

}
#endif

