/*
 *-----------------------------------------------------------------------------
 * The confidential and proprietary information contained in this file may
 * only be used by a person authorised under and to the extent permitted
 * by a subsisting licensing agreement from Arm Limited or its affiliates.
 *
 *            (C) COPYRIGHT 2010-2013 Arm Limited or its affiliates.
 *                ALL RIGHTS RESERVED
 *
 * This entire notice must be reproduced on all copies of this file
 * and copies of this file may only be made by a person if such person is
 * permitted to do so under the terms of a subsisting license agreement
 * from Arm Limited or its affiliates.
 *
 *      SVN Information
 *
 *      Checked In          : $Date: 2017-10-10 15:55:38 +0100 (Tue, 10 Oct 2017) $
 *
 *      Revision            : $Revision: 371321 $
 *
 *      Release Information : Cortex-M System Design Kit-r1p1-00rel0
 *-----------------------------------------------------------------------------
 */

//==============================================================================
//
// Cortex-M0+ System Design Kit vector table relocation test
//
//==============================================================================
//
// Simple example test case which will relocate vector table and checks that
// when an interrupt is triggered it enters the correct relocated vector handler
// region
//
// NOTE:
//
// 1. Parts of this test rely on there being some cycle delay between the act of generating
//    an interrupt (for example by writing to a GPIO output), and the interrupt being
//    driven into the processor.
//
//    If there are no delays in your system, you may need to modify parts of this test.
//

#ifdef CORTEX_M0
#include "CMSDK_CM0.h"
#include "core_cm0.h"
#endif

#ifdef CORTEX_M0PLUS
#include "CMSDK_CM0plus.h"
#include "core_cm0plus.h"
#endif

#include <stdio.h>
#include "uart_stdout.h"

#include "config_id.h"

// Test memory location for vector table relocation
#define VTOR_TEST_ADDR 0x20001100

//=====================================================================
// Function prototype
//=====================================================================
void NonMaskableInt_Handler(void);
void watchdog_unlock(void);            // unlock watchdog
void watchdog_lock(void);              // lock watchdog


/* Macros for word access */
#define HW32_REG(ADDRESS)  (*((volatile unsigned long  *)(ADDRESS)))

//=====================================================================
// Global Variables to track test progress
//=====================================================================
         uint32_t Errors = 0;   // Error counter
volatile uint32_t IntTaken = 0; // Interrupt taken

int main (void)
{
#ifdef CORTEX_M0PLUS
  volatile uint32_t testdata = 0;   // for VTOR R/W test
           uint32_t vtor_detect = 0;// indicate if VTOR is present
           int      i;              // loop variable
#endif

  // UART init
  UartStdOutInit();

#ifdef CORTEX_M0PLUS
  //-----------------------------------------------------------------
  // Check that the Vector Table Offset Register (VTOR) can be written to
  // and read back correctly. Before writing to VTOR, we need to copy
  // the vector table to SRAM.
  //-----------------------------------------------------------------

  // Banner
  puts ("Cortex-M0+ - VTOR testing\n");

  // Copy vector table to SRAM first, before carry out R/W test to VTOR
  for (i=0;i<48;i++){ // maximum number of exception is 48
    HW32_REG((VTOR_TEST_ADDR + (i<<2))) = HW32_REG((i<<2)); // Copy vector table
    }

  // Set VTOR to vector table location in SRAM
#if (__VTOR_PRESENT == 1)
  SCB->VTOR = VTOR_TEST_ADDR;
#else
  *(volatile uint32_t *)(SCS_BASE + 0x0D08UL) = VTOR_TEST_ADDR;
#endif
  __DMB();

  // read back value
  // If the VTOR is not present then the VTOR part of the SCB struct will not
  // be present so the access must be by explicit reference to the VTOR address
#if (__VTOR_PRESENT == 1)
  testdata  = SCB->VTOR;
#else
  testdata  = *(uint32_t *)(SCS_BASE + 0x0D08UL);
#endif

  printf ("  VTOR  = %x\n", testdata);

  if (testdata != VTOR_TEST_ADDR) {
    puts ("  VTOR not available\n");
    vtor_detect = 0;
    }
  else {
    vtor_detect = 1;
    }

  //-----------------------------------------------------------------
  // Check the configuration settings in CMSIS header file
  //-----------------------------------------------------------------
  if (((vtor_detect == 0) && (__VTOR_PRESENT == 1)) ||
      ((vtor_detect == 1) && (__VTOR_PRESENT == 0))) {
    printf ("ERROR: __VTOR_PRESENT (%d) does not agree with the availability of VTOR in the processor (%d)\n", __VTOR_PRESENT, vtor_detect);
    Errors++;
    }
  //-----------------------------------------------------------------
  // Check the configuration settings in config_id.h
  //-----------------------------------------------------------------
  if (((vtor_detect == 0) && (EXPECTED_VTOR == 1)) ||
      ((vtor_detect == 1) && (EXPECTED_VTOR == 0))) {
    printf ("ERROR: EXPECTED_VTOR (%d) does not agree with the availability of VTOR in the processor (%d)\n", EXPECTED_VTOR, vtor_detect);
    Errors++;
    }

  //-----------------------------------------------------------------
  // Check that vector table can be relocated at location 0x20001100
  // NMI is taken as an example to illustrate the same
  //-----------------------------------------------------------------

  if(vtor_detect == 1)
      {
        puts ("\nTest vector table relocation with NMI\n");

        // Clear IntTaken flag
        IntTaken = 0;

        // Note: Vector Table is already relocated to SRAM, so no need to change

        // Write new Handler address for NMI
	HW32_REG(( VTOR_TEST_ADDR + ((NonMaskableInt_IRQn+16)*4) )) = (unsigned long) NonMaskableInt_Handler;
        __DMB();

        // Assert NMI by driving pin on the internal GPIO
        watchdog_unlock();
        CMSDK_WATCHDOG->LOAD = 0x10000;
	// Check read back value of LOAD register
        if (CMSDK_WATCHDOG->LOAD != 0x10000) Errors ++;

        // Enable watchdog
        CMSDK_WATCHDOG->CTRL = CMSDK_Watchdog_CTRL_INTEN_Msk;

	// Check read back value of CTRL register
        if (CMSDK_WATCHDOG->CTRL != CMSDK_Watchdog_CTRL_INTEN_Msk) Errors ++;

        // Wait for Interrupt
        __WFI();

        // Check that Interrupt is triggered and got serviced
        // It has to be from new NMI handler and not from Default_IRQHandler
        if (IntTaken==1) {
	  puts ("\nVector table re-located successfully\n");
	  }
	else {
	  puts ("\nERROR: Vector table re-location failed\n");
	  Errors ++;
	  }

       // Print final test result
       if (Errors == 0)
         {
           puts("** TEST PASSED ** \n");
         }
       else
         {
           printf("** TEST FAILED ** : Test Failed with %d errors\n", Errors);
         }
      }
  else
      {
        puts ("** TEST SKIPPED ** Core does not support VTOR, skipping test\n");
      }

#else
  puts ("** TEST SKIPPED ** Core does not support VTOR, skipping test\n");
#endif

  // End simulation
  UartEndSimulation();

  return Errors;
}


//-----------------------------------------------------------------
// Functions and Handlers
//-----------------------------------------------------------------


//=====================================================================
// Replacement NMI Handler
// This should be executed when watchdog interrupt is triggered.
void NonMaskableInt_Handler(void)
{

  // Disable watchdog interrupt output
  CMSDK_WATCHDOG->CTRL = 0;   // Disable interrupt generation
  CMSDK_WATCHDOG->INTCLR = CMSDK_Watchdog_INTCLR_Msk; // Clear interrupt

  puts ("[NMI]\n");

  if (CMSDK_WATCHDOG->RAWINTSTAT != 0) {
    Errors++;
    puts ("ERROR: Cannot clear watchdog interrupt status.");
    // Disconnect interrupt generation using integration test registers
    // if watchdog interrupt cannot be cleared
    CMSDK_WATCHDOG->ITOP = 0; // IRQ and reset output = 0
    CMSDK_WATCHDOG->ITCR = 1; // Enable  integration test mode
    }

  // Signal to main code that the correct NMI was taken
  IntTaken = 1;
}

//=====================================================================
// Default NMI Handler as defined in CMSIS
// This should not be executed because the vector table is changed to
// use NonMaskableInt_Handler.
//
void NMI_Handler(void)
{
  watchdog_unlock();
  CMSDK_WATCHDOG->CTRL = 0;  // Disable interrupt generation
                             // and clear NMI generation
  CMSDK_WATCHDOG->INTCLR = CMSDK_Watchdog_INTCLR_Msk;
  watchdog_lock();

  // Default NMI handler defined by CMSIS
  puts ("ERROR: NMI Vector modification failed.");
  puts ("       Default NMI handler executed");

  // Signal to main code that the INT was taken incorrectly
  Errors++;

}

//=====================================================================
void HardFault_Handler()
{
  //
  // The HardFault Handler is only reached if there is a problem
  //
  puts ("Unexpected HardFault - FAIL\n");

  // End simulation
  UartEndSimulation();

}


//=====================================================================
// Unlock watchdog access
void watchdog_unlock(void)
{
  CMSDK_WATCHDOG->LOCK = 0x1ACCE551;
}
//=====================================================================
// Lock watchdog access
void watchdog_lock(void)
{
  CMSDK_WATCHDOG->LOCK = 0;
}

//=====================================================================

void Default_IRQHandler()
{
  // Clear NMI generation by clearing GPIO2 output

  watchdog_unlock();
  CMSDK_WATCHDOG->CTRL = 0;  // Disable interrupt generation
                             // and clear NMI generation
  CMSDK_WATCHDOG->INTCLR = CMSDK_Watchdog_INTCLR_Msk;
  watchdog_lock();

  puts ("ERROR : Default IRQ handler executed unexpectedly\n");

  // Signal to main code that the INT was taken incorrectly
  Errors++;
}

//=====================================================================
